@@ -2,7 +2,7 @@ import QuantLib as ql
 from numpy import datetime64 as dt64, datetime_as_string as datestr
 from typing import List
 from . import pymodels, ql_enums as qle, ql_utils as qlu, \
-    ql_conventions as qlc
+    ql_conventions as qlc, bonds, termstructure as ts
 
 
 class FixedBondVal:
@@ -16,7 +16,80 @@ class FixedBondVal:
     @bond_info.setter
     def bond_info(self, bond_info):
         self.__bond_info = bond_info
+
+
+class FixedRateBond:
+    def __init__(self, date: str, fixbond: pymodels.FixedRateBond, qlcalendar=None):
+        self.__fixbond = fixbond
+        if qlcalendar:
+            self.__qlcalendar = qlcalendar
+        else:
+            self.__qlcalendar = ql.NullCalendar()   
         
+        self.__sys_date = ql.Settings.instance().evaluationDate
+        ql.Settings.instance().evaluationDate = ql.Date(date,'%Y-%m-%d')
+        structures = bonds.calc_fixbond_structures(date, self.__fixbond, 
+                                                    self.__fixbond.settings, 
+                                                    self.__qlcalendar)
+        self.__fixbond.structure = structures
+
+    
+    def stats(self, clean_price: float=None, bond_yield: float=None, 
+                curve: ql.QuantLib.YieldTermStructure = None,
+                spreads: List[pymodels.Rate] = None): 
+
+        vdate = ql.Settings.instance().evaluationDate
+        setting = self.__fixbond.settings
+        qlfixratebond = bonds.get_qlfixedratebond(self.__fixbond, setting, self.__qlcalendar)
+
+        if clean_price or bond_yield:
+            if clean_price:
+                clean_price = float(clean_price)
+                ytm = bonds.fixbond_ytm(clean_price, 
+                                        setting.day_count, 
+                                        setting.frequency, 
+                                        qlfixratebond)
+            else:
+                ytm = bond_yield
+        
+            ql_ir = ql.InterestRate(float(ytm)/100, 
+                                qlc.ql_day_count[setting.day_count], 
+                                ql.Compounded, 
+                                qlc.ql_frequency[setting.frequency])
+            
+            risks = bonds.bond_functions_risks(qlfixratebond, ql_ir)
+            risks.ytm = ytm
+            return risks
+
+        elif curve:
+            curve_handle = ql.YieldTermStructureHandle(curve)
+            curve_engine = ql.DiscountingBondEngine(curve_handle)
+            qlfixratebond.setPricingEngine(curve_engine)
+            value = qlfixratebond.NPV()
+            accrued = qlfixratebond.accruedAmount()
+            clean_price = value - accrued
+            ytm  = qlfixratebond.bondYield(clean_price,
+                                    qlc.ql_day_count[setting.day_count],
+                                    ql.Compounded,
+                                    qlc.ql_frequency[setting.frequency],
+                                    vdate) * 100
+            ql_ir = ql.InterestRate(float(ytm)/100, 
+                                qlc.ql_day_count[setting.day_count], 
+                                ql.Compounded, 
+                                qlc.ql_frequency[setting.frequency])
+            risks = bonds.bond_functions_risks(qlfixratebond, ql_ir)
+            if spreads:
+                spread_handle = ts.ql_spread_handle(vdate.ISO(),
+                                    curve, 
+                                    spreads)
+                spread_engine = ql.DiscountingBondEngine(spread_handle)
+                qlfixratebond.setPricingEngine(spread_engine)
+                risks.value_after_shift = qlfixratebond.NPV()
+            
+                
+            return risks
+
+
 
 class ConvertibleFixedRateBond:
     def __init__(self, thebond: pymodels.ConvertibleFixedRate, qlcalendar):
@@ -493,7 +566,6 @@ class Curve:
             helpers += self.__fra.helpers()
         if self.__ois:
             helpers += self.__ois.helpers()
-
         if self.__irs:
             helpers += self.__irs.helpers()
 
